﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Text.RegularExpressions;

/// <summary>
/// 自動車制御クラス.
/// </summary>
public class CarController : MonoBehaviour {

    [SerializeField]
    float m_accelSpec = 5f;     // 加速力

    [SerializeField]
    float m_maxForce = 100f;    // 前に進ませる力の上限

    [SerializeField]
    float m_corneringSpec = 5f; // 1秒に切れるハンドル角

    [SerializeField]
    float m_maxCornering = 50f; // 最高ハンドル切れ角


    float m_force = 0f;         // 前に進ませる力
    float m_steering = 0;       // 現在のハンドル切れ角

    bool m_forward = false;     // アクセルを踏んでいるか
    bool m_back = false;        // ブレーキを踏んでいるか
    bool m_left = false;        // 左にステアリングを切っているか 
    bool m_right = false;       // 右にステアリングを切っているか 


    Rigidbody m_rigidBody;      // リジッドボディ


	int m_rankScore = 0;		// ランク付け用スコア
	int m_checkPointNo = 0;		// 次のチェックポイントの番号(順番通りに通過した時のみスコアをアップさせるため)
	int m_checkPointNum = 7;	// チェックポイント数(今回は決め打ち)
	float m_checkPointPassingTime = 0;	// チェックポイント通過タイム

	// Use this for initialization
	void Start () {
        // 初期化処理としてRigidobodyコンポーネントを受け取っておく
        m_rigidBody = GetComponent<Rigidbody>();
	}
	
	// Update is called once per frame
	void Update () {

        // ステアリング
        Steering();


        // ペダル操作
        Pedal();


        // 車体が向いている方向に力のベクトルを向けていく
        Vector3 targetVector = transform.forward;               // 車体の向き
        float magunitude = m_rigidBody.velocity.magnitude;      // 車体にかかっている力の強さ
        targetVector.y = m_rigidBody.velocity.y / magunitude;   // 車体にかかっている下向きの力
        // 現在かかっている力を、車体の向いている方向に近づける
        m_rigidBody.velocity = Vector3.Lerp(m_rigidBody.velocity,
                                            targetVector * magunitude,
                                            0.5f * Time.deltaTime);


        // 操作を初期化
        m_forward = false;
        m_back = false;
        m_left = false;
        m_right = false;
	}


    /// <summary>
    /// ステアリング操作.
    /// </summary>
    void Steering()
    {
        if (m_left)
        {
            m_steering -= m_corneringSpec * Time.deltaTime;
        }
        else if (m_right)
        {
            m_steering += m_corneringSpec * Time.deltaTime;
        }
        else
        {
            m_steering = Mathf.Lerp(m_steering, 0, 0.1f);
        }

        m_steering = Mathf.Clamp(m_steering, -m_maxCornering, m_maxCornering);

        Vector3 rot = transform.rotation.eulerAngles;
        rot.y += m_steering;
        transform.rotation = Quaternion.Euler(rot);

    }

    /// <summary>
    /// ペダル操作.
    /// </summary>
    void Pedal()
    {
        // アクセル
        if (m_forward)
        {
            m_force += m_accelSpec * Time.deltaTime;
        }
        // ブレーキ（バック）
        else if (m_back)
        {
            m_force -= m_accelSpec * Time.deltaTime;
        }
        // 減速
        else
        {
            m_force = Mathf.Lerp(m_force, 0, 0.2f * Time.deltaTime);
        }

        // 最高速度キャップ
        m_force = Mathf.Clamp(m_force, -m_maxForce, m_maxForce);

        // 現在速度から移動を反映
        //transform.position += transform.forward * m_speed * Time.deltaTime;
        m_rigidBody.AddForce(transform.forward * m_force * Time.deltaTime, ForceMode.Acceleration);
    }



    /// <summary>
    /// 前進操作.
    /// </summary>
    /// <value><c>true</c> if forward; otherwise, <c>false</c>.</value>
    public bool Forward
    {
        set
        {
            m_forward = value;
        }
    }


    /// <summary>
    /// 後退操作.
    /// </summary>
    /// <value><c>true</c> if back; otherwise, <c>false</c>.</value>
    public bool Back
    {
        set
        {
            m_back = value;
        }
    }


    /// <summary>
    /// 左に曲がる操作.
    /// </summary>
    /// <value>The left.</value>
    public bool Left
    {
        set
        {
            m_left = value;
        }
    }


    /// <summary>
    /// 右に曲がる操作.
    /// </summary>
    /// <value><c>true</c> if right; otherwise, <c>false</c>.</value>
    public bool Right
    {
        set
        {
            m_right = value;
        }
    }


	/// <summary>
	/// ランクスコア.
	/// </summary>
	/// <value>The rank score.</value>
	public int RankScore
	{
		get
		{
			return m_rankScore;
		}
	}


	/// <summary>
	/// チェックポイント通過タイム.
	/// </summary>
	/// <value>The check point passing time.</value>
	public float CheckPointPassingTime
	{
		get
		{
			return m_checkPointPassingTime;
		}
	}


	/// <summary>
	/// チェックポイント通過判定
	/// </summary>
	/// <param name="col">Col.</param>
	private void OnTriggerEnter(Collider other)
	{
		if (other.gameObject.tag == "CheckPoint")
		{
			// ぶつかった相手の名前の最後の文字を取り出す
			string strNo = Regex.Replace (other.gameObject.name, @"[^0-9]", "");
			// その文字を数字に変換
			int no = int.Parse (strNo);

			// 次のチェックポイントに入ったなら
			if (no == m_checkPointNo) {

				m_rankScore++;	// ランクスコアを更新

				// チェックポイント通過タイムを記録
				m_checkPointPassingTime = Time.time;

				// チェックポイント番号を増加
				m_checkPointNo++;

				// 1周したらチェックポインント番号を0にする
				if (m_checkPointNo >= m_checkPointNum) {
					m_checkPointNo = 0;
				}
			}
		}
	}
}
